<?php
namespace Teknisa\CMP\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\CMP\Services\ApprovalWorkflowService;
use Teknisa\CMP\Controllers\SendEmailController;

class ApprovalWorkflowController {
    private $baixaAtual;
    
    public function __construct(ApprovalWorkflowService $service, SendEmailController $sendEmail) {
        $this->service             = $service;
        $this->SendEmailController = $sendEmail;
    }

    public function checkApprovalWorkflow(DTO\Request\Row $request, DTO\Response $response) {   
        $row = $request->getRow();
        
        $UNIDADE_LANC    = $row['CDFILLANC'];
        $UNIDADE_DESTINO = $row['CDFILDEST'];
        $NRDEMANDA       = $row['NRDEMANDA'];
        $OPERACAO        = $row['OPERACAO'];
        $IDTPDEMANDA     = $row['IDTPDEMANDA'];

        try {
            switch($IDTPDEMANDA) {
                case 'AF':
                    if ($OPERACAO == 'LANCAMENTO') {
                        $dadosAF   = self::findDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA);
                        
                        if (!self::participaWorkflow($dadosAF, $IDTPDEMANDA)) {
                            /**
                             * Demandas que não participam do Workflow também devem
                             * passar pelo CONSISTE_ORCAMENTO_GOP para que o orçamento seja
                             * recalculado.
                             */
                            self::consisteOrcamento($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA);
                            
                            return $response->addMessage(new Message('NAO_PARTICIPA_WORKFLOW'));
                        }
                        
                        $this->baixaAtual = $this->service->getBaixaAtualAF($NRDEMANDA)['IDBAIXAAF'];
                        
                        if ($this->service->verificaAlcada($UNIDADE_DESTINO)['FILALCADAS'] > 0) {
                            $stForaOrc = self::consisteOrcamento($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA)['P_CONSISTE'];
                            
                            if($stForaOrc == 'N') {
                                $params = $this->service->getTipoOrcamento($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA);
                            }
                            else {
                                $params = $this->service->getDemandaDespesa($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA);
                                if($params['CDDESPPREV'] == null || $params['CDDESPPREV'] == '') {
                                    $params = $this->service->getDemandaConsumo($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA);
                                }
                            }
                            $nrAlcada  = $this->service->findAlcadaLancamento($UNIDADE_DESTINO, $UNIDADE_LANC, $stForaOrc, $NRDEMANDA, $dadosAF['LANCAMENTO'], $params['NMITMORCA'], $params['CDDESPPREV']);
                            if (empty($nrAlcada)) {
                                $situacao = 'SEM_ALCADA';
                                
                                /**
                                 * 18/06/2020
                                 * Sapore - caso a aprovação tenha sido recusada e, ao ser editada, e não ultrapassou o orçamento, ou seja
                                 * fica sem alçada, será considerada como APROVADA, e seu histórico de aprovações deve ser removido.
                                 */
                                if ($this->baixaAtual == 5) {
                                    $situacao = 'APROVADA';
                                    $this->baixaAtual = null;
                                }
                                
                                self::updateDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA, $dadosAF['LANCAMENTO'], $situacao, null);
                                
                                /**
                                 * 05/06/2020
                                 * Sapore - caso a aprovação tenha sido recusada e, ao ser editada, foi aprovada sem a necessidade, seu histórico de aprovações
                                 * deve ser removido.
                                 */
                                self::deletaHistoricoAprovacoesExistentes(
                                    $UNIDADE_DESTINO,
                                    $UNIDADE_LANC,
                                    $NRDEMANDA
                                );
                                
                                $response->addMessage(new Message($situacao === 'APROVADA' ? 'APROVADA_SEM_ALCADA' :  'SEM_ALCADA'));
                            }
                            else {
                                self::updateDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA, $dadosAF['LANCAMENTO'],'PENDENTE', $nrAlcada[0]['NRCONFIGALCADA']);
                                
                                /**
                                 * Alteração realizada no dia 22/11/2019 a pedido da Sapore para que somente esteja pendente
                                 * caso o orçamento tenha sido ultrapassado.
                                 * 
                                 * Se o retorno for 'N', quer dizer que ultrapassou o orçamento.
                                 * 
                                 * Trecho de código anterior:
                                 * $total  = self::getTotal($row, $nrAlcada[0]);
                                 * $total += $dadosAF['VRTOTALAF'];
                                 * 
                                 * if ($total <= $aprovador['VRAPROVCMP']) {
                                 */
                                 
                                $numeroConfigAlcada = $nrAlcada[0]['NRCONFIGALCADA'];
                                $aprovador  = $this->service->findAprovador($numeroConfigAlcada, null, $UNIDADE_DESTINO);
                                
                                if ($stForaOrc == 'S') {
                                    self::updateDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA, $dadosAF['LANCAMENTO'], 'APROVADA', $nrAlcada[0]['NRCONFIGALCADA']);
                                    
                                    /**
                                     * 05/06/2020
                                     * Sapore - caso a aprovação tenha sido recusada e, ao ser editada, foi aprovada sem a necessidade, seu histórico de aprovações
                                     * deve ser removido.
                                     */
                                    self::deletaHistoricoAprovacoesExistentes(
                                        $UNIDADE_DESTINO,
                                        $UNIDADE_LANC,
                                        $NRDEMANDA
                                    );
                                    
                                    $response->addMessage(new Message('APROVADA'));
                                } else {
                                    $operadorLanc = $this->service->valida_Oper($dadosAF['CDOPERLANCAF'], $UNIDADE_DESTINO, $numeroConfigAlcada);
                                    $IDSITUACAO = $operadorLanc == $aprovador['CDOPERADOR'] ? 'APROVADA' : 'PENDENTE';
                                    self::gera_Aprovacao($UNIDADE_DESTINO, $UNIDADE_LANC, $numeroConfigAlcada, $NRDEMANDA, $aprovador['CDOPERADOR'], $IDTPDEMANDA, $stForaOrc, $OPERACAO, $IDSITUACAO, $dadosAF['IDAUTOFORNEX'], $aprovador, $dadosAF['CDOPERLANCAF']);
                                    $response->addMessage(new Message('PENDENTE'));
                                }
                            }
                        }
                        else {
                            self::updateDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA, $dadosAF['LANCAMENTO'],'APROVADA', '');
                            $response->addMessage(new Message('APROVADA'));
                        }
                        
                        self::consisteOrcamento($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA);
                    }
                    else if ($OPERACAO == 'REVALIDA') {
                        if ($this->service->verificaAlcada($UNIDADE_LANC)) {
                            $dadosAF = self::findDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA);
                            if (empty($dadosAF['NRCONFIGALCADA'])) {
                                $nrAlcada = $this->findAlcada($UNIDADE_DESTINO, $stForaOrc, $OPERACAO, $dadosAF['IDTIPOLANC'], $dadosAF['IDAFEXTRA'], $dadosAF['LANCAMENTO']);
                                if (empty($nrAlcada)) {
                                    $response->addMessage(new Message('SEM_ALCADA'));
                                }
                                else {
                                    self::updateDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA, $dadosAF['LANCAMENTO'],'PENDENTE', $nrAlcada[0]['NRCONFIGALCADA'] );
                                    $aprovador    = $this->service->findAprovador($nrAlcada[0]['NRCONFIGALCADA'], null, $UNIDADE_DESTINO);
                                    $operadorLanc = $this->service->valida_Oper($CDOPERALTERACAO, $UNIDADE_DESTINO, $nrAlcada[0]['NRCONFIGALCADA']);
                                    $IDSITUACAO   = $operadorLanc == $aprovador['CDOPERADOR'] ? 'APROVADA' : 'PENDENTE';
                                    self::gera_Aprovacao($UNIDADE_DESTINO, $UNIDADE_LANC, $NRCONFIGALCADA, $NRDEMANDA, $CDOPERADOR, $IDTPDEMANDA, $IDCLASSIFIC, $IDOPERACAOALC, $IDSITUACAO, $dadosAF['IDAUTOFORNEX'], $aprovador, $dadosAF['CDOPERLANCAF']);
                                    $response->addMessage(new Message('PENDENTE'));
                                }
                            }
                            else {
                                $response->addMessage(new Message('VALIDO')); 
                            }
                        }
                        else {
                            $response->addMessage(new Message('VALIDO')); 
                        }
                    }
                    else {
                        if ($this->service->verificaAlcada($UNIDADE_LANC)) {
                            $dadosAF = self::findDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA);
                            if (empty($dadosAF['NRCONFIGALCADA'])) {
                                if(empty($this->valida_Alcada($dadosAF['NRCONFIGALCADA'], $OPERACAO))) {
                                    $response->addMessage(new Message('VALIDO'));     
                                }
                                else {
                                    $aprovador    = $this->service->findAprovador($dadosAF['NRCONFIGALCADA'], null, $UNIDADE_DESTINO);
                                    $operadorLanc = $this->service->valida_Oper($CDOPERALTERACAO, $UNIDADE_DESTINO, $dadosAF['NRCONFIGALCADA']);
                                    $IDSITUACAO = $operadorLanc == $aprovador['CDOPERADOR'] ? 'APROVADA' : 'PENDENTE';
                                    self::gera_Aprovacao($UNIDADE_DESTINO, $UNIDADE_LANC, $NRCONFIGALCADA, $NRDEMANDA, $CDOPERADOR, $IDTPDEMANDA, $IDCLASSIFIC, $IDOPERACAOALC, $IDSITUACAO, $dadosAF['IDAUTOFORNEX'], $aprovador, $dadosAF['CDOPERLANCAF']);
                                    $response->addMessage(new Message('PENDENTE'));
                                }
                            }
                            else {
                                $response->addMessage(new Message('VALIDO')); 
                            }
                        }
                        else {
                            $response->addMessage(new Message('VALIDO')); 
                        }
                    }
                break;
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function consisteOrcamento($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA) {
        return $this->service->executeCONSISTE_ORCAMENTO($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA);
    }
    
    public function findDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA) {
        if($IDTPDEMANDA == 'AF') return $this->service->findDemandaAF($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA);
    }
    
    public function updateDemanda($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTPDEMANDA, $IDTIPOLANC, $SITUACAO, $NRCONFIGALCADA) {
        if($IDTPDEMANDA == 'AF') {
            $this->service->updateDemandaAF($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTIPOLANC, $SITUACAO, $NRCONFIGALCADA, $this->baixaAtual);
        }
    }
    
    public function getTotalDia($dadosDemanda)
    {
        if ($dadosDemanda['IDTPDEMANDA'] == 'AF') {
            return $this->service->totalDiaAF($dadosDemanda)[0]['TOTAL'];
        }
    }
    
    public function gera_Aprovacao($UNIDADE_DESTINO, $UNIDADE_LANC, $NRCONFIGALCADA, $NRDEMANDA, $CDOPERADOR, $IDTPDEMANDA, $IDCLASSIFIC, $IDOPERACAOALC, $IDSUTACAO, $IDAUTOFORNEX, $aprovador, $CDOPERLANCAF) {
        if($IDCLASSIFIC == 'S') {
            $IDCLASSIFIC = 'FORA_ORCAMENTO';
        }
        else {
            if($IDAUTOFORNEX == 'EXTRA') {
                $IDCLASSIFIC = 'EXTRA';
            }
            else {
                $IDCLASSIFIC = 'PADRAO';
            }
        }
        
        self::deletaHistoricoAprovacoesExistentes(
            $UNIDADE_DESTINO,
            $UNIDADE_LANC,
            $NRDEMANDA
        );
        
        $this->service->inserirRegistroAprov($UNIDADE_DESTINO, $UNIDADE_LANC, $NRCONFIGALCADA, $NRDEMANDA, $CDOPERADOR, $IDTPDEMANDA, $IDCLASSIFIC, $IDOPERACAOALC, $IDSUTACAO);
        
        $operadorLanc = $this->service->getOperLancamento($CDOPERLANCAF);
        $emails = $this->service->getOperAddress($aprovador['NRALCADAGERFILI']);
        $this->sendEmail($UNIDADE_DESTINO, $NRCONFIGALCADA, $IDTPDEMANDA, $NRDEMANDA, $IDCLASSIFIC, $operadorLanc, $emails);
    
    }

    public function sendEmail($UNIDADE_DESTINO, $NRCONFIGALCADA, $IDTPDEMANDA, $NRDEMANDA, $IDCLASSIFIC, $operadorLanc, $emails) {
        
        foreach ($emails as $operAddress) {
            $this->SendEmailController->sendMail(
                array(
                    "MOTIVO"           => 'APROVACAO_ALCADA',
                    "NMOPERADOR_FROM"  => $operadorLanc['NMOPERADOR'],
                    "DSEMAILOPER_FROM" => $operadorLanc['DSEMAILOPER'],
                    "NMOPERADOR"       => $operAddress['NMOPERADOR'],
                    "DSEMAILOPER"      => $operAddress['DSEMAILOPER'],
                    "IDTPDEMANDA"      => $IDTPDEMANDA,
                    "NRAUTOFORN"       => $NRDEMANDA,
                    "NMFILIAL"         => $this->service->getNMFILIAL($UNIDADE_DESTINO),
                    "NMALCADA"         => $this->service->getNMALCADA($NRCONFIGALCADA),
                    "IDCLASSIFIC"      => $IDCLASSIFIC
                )
            );
        }
        
    }
    
    public function getTotal($dadosDemanda, $alcada)
    {
        $paramValorUtilizado = self::getParamConsisteValorAprovacao($dadosDemanda, $alcada);
        
        if ($paramValorUtilizado == 'M') {
            $dataEntregaAF    = $this->getDataEntregaAF($dadosDemanda);
            $periodoOrcamento = $this->getPeriodoOrcamento($dadosDemanda, $dataEntregaAF);
            
            return self::getTotalPeriodo($dadosDemanda, $periodoOrcamento, $dataEntregaAF);
        }
        
        return self::getTotalDia($dadosDemanda);
    }
    
    public function getTotalPeriodo($dadosDemanda, $periodoOrcamento, $dataEntregaAF)
    {
        if ($dadosDemanda['IDTPDEMANDA'] == 'AF') {
            $paramsTotalPeriodo = [
                'DTPROGENTAF' => $dataEntregaAF,
                'CDFILDEST'   => $dadosDemanda['CDFILDEST'],
                'NRDEMANDA'   => $dadosDemanda['NRDEMANDA'],
                'CDARVPROD'   => $this->getCodigoProdutoOrcamento($dadosDemanda, $periodoOrcamento, $dataEntregaAF),
            ];
            
            return $this->service->getTotalPeriodoAF($paramsTotalPeriodo)[0]['VR_TOTAL_AF'];
        }
    }
    
    public function getCodigoProdutoOrcamento($dadosDemanda, $periodoOrcamento, $dataEntregaAF)
    {
        $paramsFindCodigoProdutoOrcamento = [
            'IDPERIODOORCA' => $periodoOrcamento['V_IDPERIODOORCA'],
            'NRPERIODOORCA' => $periodoOrcamento['V_NRPERIODOORCAGOP'],
            'DTPROGENTAF'   => $dataEntregaAF,
            'CDFILDEST'     => $dadosDemanda['CDFILDEST'],
            'CDFILLANC'     => $dadosDemanda['CDFILLANC'],
            'NRDEMANDA'     => $dadosDemanda['NRDEMANDA'],
        ];
            
        $codigoProdutoOrcamento = $this->service->findCodigoProdutoOrcamento($paramsFindCodigoProdutoOrcamento);
    }
    
    public function getDiaInicioOrcamento($dadosDemanda)
    {
        return $this->service->findDiaInicioOrcamento($dadosDemanda['CDFILDEST'])[0]['DIAINIORCA'];
    }
    
    public function getPeriodoOrcamento($dadosDemanda, $dataEntregaAF)
    {
        $diaInicioOrcamento = $this->getDiaInicioOrcamento($dadosDemanda);
        
        $dataOrcamento = $this->getDataOrcamento($diaInicioOrcamento, $dataEntregaAF);
        
        $paramsPeriodoOrcamento = [
            'V_DATAINI' => $dataOrcamento['V_DATAINI'],
            'DIAINIORCA' => $diaInicioOrcamento,
        ];
        
        return $this->service->findPeriodoOrcamento($paramsPeriodoOrcamento)[0];
    }
    
    public function getDataOrcamento($diaInicioOrcamento, $dataEntregaAF)
    {
        $params = [
            'DIA' => $diaInicioOrcamento,
            'DATA' => $dataEntregaAF,
        ];
        
        return $this->service->findDataOrcamento($params)[0]; 
    }
    
    public function getDataEntregaAF($params)
    {
        return $this->service->findDataEntregaAF($params)[0]['DTPROGENTAF'];
    }
    
    public function getParamConsisteValorAprovacao($dadosDemanda, $alcada)
    {
        $params = [
            'CDFILDEST'      => $dadosDemanda['CDFILDEST'],
            'NRCONFIGALCADA' => $alcada['NRCONFIGALCADA'],
        ];
        
        return $this->service->findFilialAlcada($params)[0]['IDCONSISTVRAPROV'];
    }
    
    public function participaWorkflow($demanda, $tipoDemanda)
    {
    	if ($tipoDemanda == 'AF') {
    		$validations = [
    			'Consistente'      => $demanda['IDSITUATUAAF'] == '0',
    			'Dentro do Limite' => $demanda['IDAFFORALIM'] == 'N',
    			'Baixa Permitida'  => function($demanda) {
    			    $baixasPermitidas = [
            			'1', // Baixa Parcial
            			'0', // Em Aberto
            		];
    			    $existsAprovacaoRecusada = $this->service->findAprovacaoRecusada($demanda);
    			    
    			    if (empty($existsAprovacaoRecusada)) {
    			        return in_array($demanda['IDBAIXAAF'], $baixasPermitidas);
    			    }
    			    
    			    return true;
    			},
    		];
    		
    		foreach ($validations as $validation => $validationResult) {
    		    if (is_callable($validationResult)) {
    		        $validationResult = call_user_func($validationResult, $demanda);
    		    }
    		    
    		    if (!$validationResult) {
    		        return false;
    		    }
    		}
    	}
    	
    	return true;
    }
    
    public function deletaHistoricoAprovacoesExistentes(
        $unidadeDestino,
        $unidadeLancamento,
        $numeroDemanda
    ) {
        $registrosAprovacao = $this->service->verificaRegistroAprov(
            $unidadeDestino,
            $unidadeLancamento,
            $numeroDemanda
        );
        
        if (count($registrosAprovacao)) {
            $this->service->deletarAprovacoesExistentes($numeroDemanda);
        }
    }
}
