<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\AutorizacaoFornecimento\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\CMP\Util\CmpUtilities;
use Teknisa\CMP\Util\Attachment;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;

class Cmp03900LancamentoAutorizacaoDeFornecimentoControllerReport extends \Zeedhi\Framework\Controller\Report 
{
    protected $reportService;
    protected $qr2Strategy;
    protected $connection;
    protected $service;
    protected $cmpUtilities;
    protected $stNomeRel;
    protected $idsisdos;
    protected $logEnvioAF;
    
    protected $globalReports = [];

    public function __construct($reportService, $qr2Strategy, $connection, $service, $birtStrategy, CmpUtilities $cmpUtilities) 
    {
        $this->reportService = $reportService;
        $this->qr2Strategy   = $qr2Strategy;
        $this->connection    = $connection;
        $this->service       = $service;
        $this->birtStrategy  = $birtStrategy;
        $this->cmpUtilities  = $cmpUtilities;
        $this->logEnvioAF    = array('REPORT' => array(), 'SUCCESS' => array(), 'ERROR' => array());
    }
    
    protected $reportMapping = array(
        'padraoEnvioAF' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
                array (
                    'CDFILIAL'       => 'P_CDFILIAL',
                    'NRAUTOFORN'     => 'P_NRAUTOFORN',
                    'CDFILLANC'      => 'P_CDFILLANC',
                    'IMPRIMEOBS'     => 'P_IDEXIBEOBSFIL',
                    'IMPRIMEOBSPROD' => 'P_IMPRIMEOBSPROD',
                    "NRORG"          => 'NRORG',
                ),
            ),
    );
    
    public function geraRelatorio(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
            $this->stNomeRel = $this->getReportName($row);
            
            if (!array_key_exists($this->stNomeRel, $this->reportMapping)) {
                $this->reportMapping[$this->stNomeRel] =
                    array (
                    'strategy' => $row['STRATEGY'],
                    'parameterMapping' => 
                        array (
                            'CDFILIAL'       => 'P_CDFILIAL',
                            'NRAUTOFORN'     => 'P_NRAUTOFORN',
                            'CDFILLANC'      => 'P_CDFILLANC',
                            'PROXMES'        => 'P_PROXMES',
                            'IMPRIMEOBS'     => 'P_IMPRIMEOBS',
                            'IMPRIMEOBSPROD' => 'P_IMPRIMEOBSPROD',
                            'NRMENSAUTFOR'   => 'L_NRMENSAUTFOR',
                            "NRORG"          => 'P_NRORG',
                        )
                    );
            }
            
            $reportURL = parent::processReport($request);
            $response->addMessage(new DTO\Response\Message($reportURL));
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function geraRelatEmail(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $dadosEmail = $row['emailData'];
        $isEmailSend = false;
        $localResponse = new DTO\Response();
        
        try {
            $messages = [];
            
            if($row['IDTPENVAFNORMAL'] === 'E') {
                $row['reportLabel'] = 'padraoEnvioAF';
                $row['__report_name'] = 'padraoEnvioAF';
    
                $this->birtStrategy->setReportFormat(strtolower($dadosEmail['TYPE']));
                parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($row), "", "", ""), $localResponse);
            }
            
            $isEmailSend = self::enviarEmail($row, $dadosEmail);
            
            if($isEmailSend) {
                $this->logEnvioAF['SUCCESS'][] = array("NMFANTFORN" => $row['NMFANTFORN'], "NRAUTOFORN" => $row['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_SUCCESS");
            } else {
                $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $row['NMFANTFORN'], "NRAUTOFORN" => $row['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_FAILED");
            }
            
            $this->globalReports = [];
            
            $response->addMessage(new Message($this->logEnvioAF, 'W'));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function sendGroupMail(DTO\Request\Dataset $request, DTO\Response $response)
    {
        $data = $request->getDataSet()->getRows();
        
        if(!empty($data)) {
        
            $rows = $data[0];
            $dadosEmail = $rows['emailData'];
            $isEmailSend = false;
            $localResponse = new DTO\Response();
            
            try {
                $cdforneced = array_unique(array_map(function($row) {
                    return $row['CDFORNECED'];
                }, $rows['relatorioData']));
                
                foreach ($cdforneced as $value){
                    $params = array_filter($rows['relatorioData'], function($row) use($value) {
                        return $row['CDFORNECED'] == $value;
                    });
                    
                    $cdfildest = join("_", array_unique(array_map(function($row) {
                        return $row['CDFILDEST'];
                    }, $params)));
                    
                    $cdcomprador = join("_", array_unique(array_map(function($row) {
                        return $row['CDOPERADOR'];
                    }, $params)));
                    
                    $cdfillanc = array_unique(array_map(function($row) {
                        return $row['CDFILLANC'];
                    }, $params));
                    
                    foreach($cdfillanc as $cdfilial){
                        $row = array_pop($params);
                        $row['CDFILDEST'] = $cdfildest;
                        $row['CDFILLANC'] = $cdfilial;
                        $row['CDCOMPRADOR'] = $cdcomprador;
                        $row['NRSEQAPURAF'] = null;
                        
                        $row['reportLabel'] = 'padraoEnvioAF';
                        $row['__report_name'] = 'padraoEnvioAF';
                        
                        $this->reportMapping['padraoEnvioAF'] = array (
                            'strategy' => 'BIRT',
                            'parameterMapping' => 
                                array (
                                    'NRSEQAPURAF'    => 'P_NRSEQAPURAF',
                                    'CDFILLANC'      => 'P_CDFILLANC',
                                    'IMPRIMEOBSPROD' => 'P_IMPRIMEOBSPROD',
                                    'IMPRIMEOBS'     => 'P_IDEXIBEOBSFIL',
                                    'CDFORNECED'     => 'P_CDFORNECED',
                                    'DTENTINI'       => 'P_DTENTREGAINI',
                                    'DTENTFIN'       => 'P_DTENTREGAFIN',
                                    'CDFILDEST'      => 'P_CDFILDEST',
                                    'CDCOMPRADOR'    => 'P_CDOPERADOR',
                                    'DTAFINI'        => 'P_DTAUTOFORNINI',
                                    'DTAFFIN'        => 'P_DTAUTOFORNFIN',
                                    "NRORG"          => 'NRORG'
                                )
                            );
                        $this->birtStrategy->setReportFormat(strtolower($dadosEmail['TYPE']));
                        parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($row), "", "", ""), $localResponse);
                        $isEmailSend = self::enviarEmail($row, $dadosEmail, $cdfilial.$value);
                        
                        if($isEmailSend) {
                            $this->logEnvioAF['SUCCESS'][] = array("NMFANTFORN" => $row['NMFANTFORN'], "NRAUTOFORN" => $row['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_SUCCESS");
                        } else {
                            $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $row['NMFANTFORN'], "NRAUTOFORN" => $row['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_FAILED");//'EMAIL_SEND_FAILED';
                        }
                        
                        $this->globalReports = [];
                    }
                }
                
                $response->addMessage(new Message($this->logEnvioAF, 'W'));
                
            } catch (\Exception $e) {
                $response->setError(new Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function verificaParametrosEnvioRelatorioExtra($row)
    {
        return $row['IDAFFORALIM'] == 'N' && $row['IDSITUATUAAF'] == null && $row['IDBAIXAAF'] == null && 
            ($row['IDAFOPERLOG'] == null || $row['IDAFOPERLOG'] == '1') && 
            (($row['IDAFPRODWEB'] == 'N' || $row['IDAFPRODWEB'] == null) || 
            ($row['IDBAIXAAF'] == '2' || $row['IDBAIXAAF'] == '3'));
    }
    
    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report)
    {
        $row = $request->getRow();
        
        $this->globalReports[$row['reportLabel']] = $report;
    }
    
    public function enviarEmail($row, $dadosEmail, $retortName = null)
    {
        if($row['IDTPENVAFFOR'] !== '1') {
            $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $row['NMFANTFORN'], "MESSAGE" => "MAIL_NOT_SENDED_IDTPENVAFFOR");
        } else {
            $this->logEnvioAF['REPORT'][] = $this->globalReports;
            
            $mensagem   = $dadosEmail['MENSAGEM'];
            $encontrar  = array("<i>[Nome Fornecedor]</i>", "<i>[link do portal]</i>", "<i>[nº do lote]</i>", "<i>[Nome Operador]</i>");
            $substituir = array($row['NMFANTFORN'], $row['DSLINKECOMMERCE'], $row['NRSEQAPURAF'], $row['NMOPERADOR']);
            
            $this->cmpUtilities->defineCookieEmail("MAIL_SUBJECT_ENVIO_AF", $dadosEmail['ASSUNTO'], time() + (86400 * 999));
            $this->cmpUtilities->defineCookieEmail("MAIL_BODY_ENVIO_AF", $mensagem, time() + (86400 * 999));
            
            $mensagem = str_replace($encontrar, $substituir, $mensagem);
            
            switch($dadosEmail["TYPE"]){
                case "PDF":
                    $type = 'application/pdf';
                    break;
                case "XLS":
                    $type = 'application/xls';
                    break;
                case "RTF":
                    $type = 'application/rtf';
                    break;
                case "TXT":
                    $type = 'application/txt';
                    break;
                default:
                    $type = 'application/txt';
            }
    		    
    	    $enderecos = explode(";", $row['DSEMAILCOT']);
            $enderecosCopia = !empty($row["DSMAILCOTCOP"]) ? explode(";", $row["DSMAILCOTCOP"]) : false;
            $subject = $dadosEmail["ASSUNTO"];
            
            $message = "<table>
                            <tr>
                                <td class='title'> 
                                    <div>
                                        <br/>" . $subject . "
                                    </div>
                                </td>
                            </tr>
                            <tr>
                                <td class='description'>
                                    <br/>" . $mensagem .
                                "</td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <center>
                                        <br> <p>" . $dadosEmail["MSG_AUTO"] . "</p>
                                    </center>
                                </td>
                            </tr>
                        </table>";
            
            $attachments = [];
            $attOnBody = '';
            $clearAttachments = true;
            
            if($row['IDTPENVAFNORMAL'] === 'E') {
                foreach ($this->globalReports as $name => $report) {
                    $fileName = ($retortName ? $retortName : $row["NRAUTOFORN"]) . '.' . strtolower($dadosEmail["TYPE"]);
                    //$fileName = $row["NRAUTOFORN"] . '.' . strtolower($dadosEmail["TYPE"]);
                    
                    $attachment = new Attachment($report, $fileName);
                    $attachment->setIsStringAttachment(true);
                    
                    $attachments[] = $attachment;
                    
                    $attOnBody.= "<a href= '".str_replace("PDF", "pdf", $report)."'>" . $dadosEmail["MSG_VISUALIZA"] . "</a><br>";
                }
                
                $attachmentBody = "<tr><td style='text-align: center;'><br>".$attOnBody."</td></tr>";
                
                $message = $attachmentBody.$message;
            }
            
            $messageSendMail = $this->cmpUtilities->messageSendMail($dadosEmail['FILIAL_LOGADA'], $dadosEmail["ASSUNTO"], $message, 'PORTAL E-COMMERCE');
            
            /**
             * 12/06/2020 - Implementação a pedido da Sapore.
             */
            if (self::enviarEmailCopiaOperadorLogado($row)) {
                $enderecosCopia[] = $this->service->getCurrentEmail();
            }
            
            if(empty($enderecos[0])) {
                $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $row['NMFANTFORN'], "MESSAGE" => "EMAIL_N_REGISTERED");
            } else {
                try{
                    return $this->cmpUtilities->enviaEmail('', $subject, $enderecos, $messageSendMail, $enderecos, $enderecosCopia, $attachments, $clearAttachments);
                } catch (\Exception $e) {
                    return false;
                }
            }
        }
    }
    
    public function getDadosEmail(DTO\Request\DataSet $request, DTO\Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        $emailRow = array();
        $relatorioRow = array();
        $result = array();
        
        if (is_array($rows)) {
            for ($i = 0; $i < count($rows); $i++) {
                $relatorioRow[$i][0]  = $rows[$i]["CDFILDEST"];
                $relatorioRow[$i][1]  = $rows[$i]["CDFILLANC"];
                $relatorioRow[$i][2]  = $rows[$i]["NRAUTOFORN"];
                $relatorioRow[$i][3]  = $rows[$i]['CDFORNECED'];
                $emailRow[$i][0] = (array_key_exists ( "NRSEQAPURAF" , $rows[$i] ) ) ? $rows[$i]["NRSEQAPURAF"] : "";
                
                $query = "SELECT
                            F.IDTPIJURFORN,
                            F.NRINSJURFORN,
                            F.CDFORNECED,
                            F.NMFANTFORN,
                            F.IDUTILZIPAF,
                            NVL(F.IDTPENVAFNORMAL, 'E') AS IDTPENVAFNORMAL,
                            F.IDTPENVAFFOR,
                            F.CDFORNERELACAF AS CDFORNERELAC,
                            NVL((SELECT LISTAGG(CASE INSTR(REPLACE(D.DSLOGINECO, ',', ';'), ';', -1, 1) WHEN LENGTH(REPLACE(D.DSLOGINECO, ',', ';')) THEN SUBSTR(REPLACE(D.DSLOGINECO, ',', ';'), 1, INSTR(REPLACE(D.DSLOGINECO, ',', ';'), ';', -1, 1) -1) ELSE REPLACE(D.DSLOGINECO, ',', ';') END, ';') WITHIN GROUP(ORDER BY D.DSEMAILECO) AS DSEMAILECO FROM DADOSFORNECOMMERCE D WHERE D.CDFORNECED = F.CDFORNECED AND D.IDACESSOAF = 'S'), DSEMAILCOT) AS DSEMAILCOT,
                            DSMAILCOTCOP
                          FROM
                           	FORNECEDOR F
                          WHERE
                           	INSTR(:P_CDFORNECED, F.CDFORNECED) > 0";
                
                $params = array(
                    "P_CDFORNECED" => $rows[$i]['CDFORNECED']
                );
                
                $stmt = $this->connection->prepare($query);
                $stmt->execute($params);
                $dadosEmail = $stmt->fetchAll();
                
                if (array_key_exists("DSEMAILCOT", $dadosEmail[0])) {
                    
                    $emailRow[$i][1] = $dadosEmail[0]["NMFANTFORN"];
                    $emailRow[$i][2] = $dadosEmail[0]["DSEMAILCOT"];
                    
                    $query = "SELECT DSLINKECOMMERCE FROM PARAMGERALECOMMERCE WHERE CDFILIAL = :P_CENTRALDECOMPRAS";
                    $params = array(
                        "P_CENTRALDECOMPRAS" => $rows[$i]['CDFORNECED']
                    );
                    
                    $stmt = $this->connection->prepare($query);
                    $stmt->execute($params);
                    $linkPortal = $stmt->fetchAll();
                    
                    $emailRow[$i][3] = (array_key_exists ( "DSLINKECOMMERCE" , $linkPortal ) ) ? $linkPortal["DSLINKECOMMERCE"] : "";
                    $emailRow[$i][5] = $dadosEmail[0]["IDTPENVAFFOR"];
                    $emailRow[$i][6] = $dadosEmail[0]["IDTPENVAFNORMAL"];
                }
                
                if (array_key_exists("DSMAILCOTCOP", $dadosEmail[0])) {
                    $emailRow[$i][4] = $dadosEmail[0]["DSMAILCOTCOP"];
                }
            }
        }
        
        $result["relatorioRow"] = $relatorioRow;
        $result["emailRow"] = $emailRow;

        $response->addDataSet(new DataSet('data', $result));
    }
    
    public function enviarEmailCopiaOperadorLogado($row)
    {
        $idsisdos = $this->service->getIdsisdos($row);
        
        if ($idsisdos == 3) {
            return false;
        }
        
        if (!$this->service->unidadeDestinoPossuiAlcada($row)) {
            return false;
        }
        
        return true;
    }
}