<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\ListaPreco\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class Cmp04502EmissaoDaListaDePrecosController extends \Zeedhi\Framework\Controller\Report
{
    protected $qr2Strategy;
    protected $environment;
    protected $connection;
    protected $cmpUtilities;
    protected $stNomeRel;

    public function __construct($reportService, $qr2Strategy, $environment, $connection, $cmpUtilities)
    {
        $this->qr2Strategy     = $qr2Strategy;
        $this->reportService   = $reportService;
        $this->environment     = $environment;
        $this->connection      = $connection;
        $this->cmpUtilities    = $cmpUtilities;
    }

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $row['CDFILCENTCMP'] = $this->cmpUtilities->getCentralCompras()['CDFILCENTCMP'];
        $row['SISTEMA']      = $this->cmpUtilities->getVisaoSistema();
        $row['CDREGIAO']     = $this->getRegiaoFromFilial($row);
        
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        
        $this->stNomeRel = $this->getReportName($row);
        $this->reportMapping[$this->stNomeRel] =
            array (
                'strategy' => $row['STRATEGY'],
                'parameterMapping' =>
                    array (
                        "CDFILIAL"          => "P_CDFILIAL",
                        "CDREGIAO"          => "P_CDREGIAO",
                        "CDFILCENTCMP"      => "P_CDFILCENTCMP",
                        "PROD"              => "P_TODOSPROD",
                        "CDPRODINI"         => "P_CDPRODINI",
                        "P_CDPRODFIN"       => "P_CDPRODFIN",
                        "COMPRADOR"         => "L_LISTAOPER",
                        "FORNECEDOR"        => "L_LISTAFORNE",
                        "COMPRADOR1"        => "L_LISTAOPER1",
                        "FORNECEDOR1"       => "L_LISTAFORNE1",
                        "SEPPAG"            => "P_PAGINA",
                        "DTINI"             => "P_DTINI",
                        "DTFIM"             => "P_DTFIN",
                        "SISTEMA"           => "P_SISTEMA",
                        "INDICADOR"         => "P_VARIPRECO",
                        "PRECOECO"          => "P_ORDPI"                      
                    ),
            );
    }
   
    public function getRegiaoFromFilial($row)
    {
        $params = array(
            "P_CDFILIAL" => $row['CDFILIAL']
        ); 
        
        $sql = "SELECT CDREGIAO, NRORG FROM FILIAL WHERE CDFILIAL = :P_CDFILIAL";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
		$result = $stmt->fetch();
		
        return $result['CDREGIAO'];
    }
}
